import openpyxl
import os
import pandas as pd
import re
from django.db.models import Q
from django.http import JsonResponse, HttpResponse
from django.shortcuts import render
from .models import Guest

# Create your views here.
def is_valid_email(email):
    email_regex = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
    return re.match(email_regex, email) is not None

def index(request):
    if request.method == 'POST':
        searchInput = request.POST.get('searchInput')

        # Initialize the guests QuerySet
        guest = Guest.objects.none()

        if is_valid_email(searchInput):
            guest = Guest.objects.filter(email__iexact=searchInput).first()
    
            if guest:
                guest_id = guest.id
        
                checkin_guest(request, guest_id)
        
                guest_data = {
                    'id': guest.id,
                    'first_name': guest.first_name,
                    'last_name': guest.last_name,
                    'email': guest.email,
                }
        
                return JsonResponse({"status": 200, "guest": guest_data})
            else:
                return JsonResponse({"status": 404})
        else:
            return JsonResponse({"status": 400})
    
    return render(request, 'index.html')


def dashboard(request):
    guests = Guest.objects
    all_guests = guests.all()
    guest_count = guests.count()
    checkedin_guest_count = guests.filter(status='validated').count()
    not_checkedin_guest_count = guests.filter(status='unvalidated').count()
    context = {
        'all_guests': all_guests,
        'guest_count' : guest_count,
        'checkedin_guest_count': checkedin_guest_count,
        'not_checkedin_guest_count': not_checkedin_guest_count
    }
    return render(request, 'dashboard.html', context=context)

def all_guests(request):
    guests = Guest.objects
    all_guests = guests.all()
    guest_count = guests.count()
    checkedin_guest_count = guests.filter(status='validated').count()
    not_checkedin_guest_count = guests.filter(status='unvalidated').count()
    context = {
        'all_guests': all_guests,
        'guest_count' : guest_count,
        'checkedin_guest_count': checkedin_guest_count,
        'not_checkedin_guest_count': not_checkedin_guest_count
    }
    return render(request, 'all_guests.html', context=context)

def checkedin_guests(request):
    guests = Guest.objects
    checkedin_guests = guests.filter(status='validated').all()
    guest_count = guests.count()
    checkedin_guest_count = guests.filter(status='validated').count()
    not_checkedin_guest_count = guests.filter(status='unvalidated').count()
    context = {
        'checkedin_guests': checkedin_guests,
        'guest_count' : guest_count,
        'checkedin_guest_count': checkedin_guest_count,
        'not_checkedin_guest_count': not_checkedin_guest_count
    }
    return render(request, 'checkedin_guests.html', context=context)

def not_checkedin_guests(request):
    guests = Guest.objects
    not_checkedin_guests = guests.filter(status='unvalidated').all()
    guest_count = guests.count()
    checkedin_guest_count = guests.filter(status='validated').count()
    not_checkedin_guest_count = guests.filter(status='unvalidated').count()
    context = {
        'not_checkedin_guests': not_checkedin_guests,
        'guest_count' : guest_count,
        'checkedin_guest_count': checkedin_guest_count,
        'not_checkedin_guest_count': not_checkedin_guest_count
    }
    return render(request, 'not_checkedin_guests.html', context=context)

def checkin_guest(request, guest_id):
    if request.method == 'POST':
        # guest_id = request.POST.get('id')
        guest = Guest.objects.get(pk=guest_id)
        if guest and guest.status == 'unvalidated':
            guest.status = 'validated'
            guest.save()
            return JsonResponse({'status': 200})
        return JsonResponse({'status': 404})
    return JsonResponse({'status': 403}) 

def import_guest_list(request):
    if request.method == 'POST' and request.FILES.get('guest_list'):
        file = request.FILES['guest_list']
        try:
            wb = openpyxl.load_workbook(file)
            sheet = wb.active
            inserted_count = 0

            for row in sheet.iter_rows(min_row=2, values_only=True):
                first_name, last_name, email = row

                if email and not Guest.objects.filter(email=email).exists():                                 
                    Guest.objects.create(
                        first_name=first_name.strip() if first_name else '',
                        last_name=last_name.strip() if last_name else '',
                        email=email.strip() if email else '',
                    )
                    inserted_count += 1
                    print(f"<{email} inserted successfully>")
                else:
                    print(f"<{email} already exists>")

            return JsonResponse({'message': f'File processed successfully. {inserted_count} records inserted.', 'status': 200}) 
        except Exception as e:
            return JsonResponse({'message': 'Something went wrong, try again.', 'status': 400, 'error': str(e)})
    
    return JsonResponse({'message': 'Invalid request or file missing.', 'status': 400})


def export_to_excel(request):
    guests = Guest.objects.all().values('first_name', 'last_name', 'email', 'status')

    df = pd.DataFrame(list(guests))
    df.columns = ['First Name', 'Last Name', 'Email', 'Status']
    response = HttpResponse(content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
    response['Content-Disposition'] = 'attachment; filename=guests_list.xlsx'
    
    df.to_excel(response, index=False)
    return response

def export_not_checkedin_to_excel(request):
    guests = Guest.objects.filter(status='no').all().values('first_name', 'last_name', 'email', 'status')

    df = pd.DataFrame(list(guests))
    df.columns = ['First Name', 'Last Name', 'Email', 'Status']
    response = HttpResponse(content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
    response['Content-Disposition'] = 'attachment; filename=guests_list.xlsx'
    
    df.to_excel(response, index=False)
    return response

def export_checkedin_to_excel(request):
    guests = Guest.objects.filter(status='yes').all().values('first_name', 'last_name', 'email', 'status')

    df = pd.DataFrame(list(guests))
    df.columns = ['First Name', 'Last Name', 'Email', 'Status']
    response = HttpResponse(content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
    response['Content-Disposition'] = 'attachment; filename=guests_list.xlsx'
    
    df.to_excel(response, index=False)
    return response

